From 93bbf0d9e79c1ed930b16a8870e640b96c65db17 Mon Sep 17 00:00:00 2001
From: Hugo Beauzee-Luyssen <beauze.h@gmail.com>
Date: Wed, 30 Sep 2009 15:28:15 +0200
Subject: [PATCH 1/2] Adding audio support to fake demux

---
 modules/access/fake.c |  109 +++++++++++++++++++++++++++++++++++++++++--------
 1 files changed, 92 insertions(+), 17 deletions(-)

diff --git a/modules/access/fake.c b/modules/access/fake.c
index 0cd8b09..e0a5f14 100644
--- a/modules/access/fake.c
+++ b/modules/access/fake.c
@@ -5,6 +5,8 @@
  * $Id$
  *
  * Author: Christophe Massiot <massiot@via.ecp.fr>
+ *         Hugo Beauzee-Luyssen <beauze.h@gmail.com>
+ *         Christophe Courtaut <christophe.courtaut@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +60,13 @@ static void Close( vlc_object_t * );
     "end-of-file (default is -1 meaning that the stream is unlimited when " \
     "fake is forced, or lasts for 10 seconds otherwise. 0, means that the " \
     "stream is unlimited).")
+#define RATE_TEXT N_("Fake audio samplerate")
+#define RATE_LONGTEXT N_( \
+    "Fake audio samplerate")
+
+#define NBSAMPLES_TEXT N_("Fake audio samples number")
+#define NBSAMPLES_LONGTEXT N_( \
+    "Fake number of audio samples")
 
 vlc_module_begin ()
     set_shortname( N_("Fake") )
@@ -68,9 +77,10 @@ vlc_module_begin ()
     add_integer( "fake-caching", DEFAULT_PTS_DELAY / 1000, NULL,
                  CACHING_TEXT, CACHING_LONGTEXT, true )
     add_float( "fake-fps", 25.0, NULL, FPS_TEXT, FPS_LONGTEXT, true )
+    add_integer( "fake-rate", 44100, NULL, RATE_TEXT, RATE_LONGTEXT, true)
     add_integer( "fake-id", 0, NULL, ID_TEXT, ID_LONGTEXT, true )
-    add_integer( "fake-duration", -1, NULL, DURATION_TEXT, DURATION_LONGTEXT,
-                 true )
+    add_integer( "fake-duration", -1, NULL, DURATION_TEXT, DURATION_LONGTEXT, true )
+    add_integer( "fake-nb-audio-samples", 512, NULL, NBSAMPLES_TEXT, NBSAMPLES_LONGTEXT, true )
 
     add_shortcut( "fake" )
     set_capability( "access_demux", 10 )
@@ -85,10 +95,18 @@ static int Control( demux_t *, int, va_list );
 
 struct demux_sys_t
 {
-    float f_fps;
-    mtime_t i_last_pts, i_duration, i_first_pts, i_end_pts, i_pause_pts;
+    float   f_fps;
+    int     i_samplerate;
+    int     i_nb_samples;
+    mtime_t i_last_pts, i_last_video_pts, i_last_audio_pts;
+    mtime_t i_duration, i_first_pts, i_end_pts, i_pause_pts;
+    mtime_t video_period;
+    mtime_t audio_period;
+    mtime_t period;
+
 
     es_out_id_t  *p_es_video;
+    es_out_id_t  *p_es_audio;
 };
 
 /*****************************************************************************
@@ -98,7 +116,10 @@ static int Open( vlc_object_t *p_this )
 {
     demux_t     *p_demux = (demux_t*)p_this;
     demux_sys_t *p_sys;
-    es_format_t fmt;
+    es_format_t fmt_video;
+    es_format_t fmt_audio;
+    bool b_audio;
+
 
     if( *p_demux->psz_access != '\0' )
     {
@@ -148,11 +169,33 @@ static int Open( vlc_object_t *p_this )
             p_sys->i_duration = 10000*1000;
     }
     p_sys->f_fps = var_CreateGetFloat( p_demux, "fake-fps" );
+    p_sys->i_samplerate = var_CreateGetInteger( p_demux, "fake-rate" );
 
     /* Declare the elementary stream */
-    es_format_Init( &fmt, VIDEO_ES, VLC_FOURCC('f','a','k','e') );
-    fmt.i_id = var_CreateGetInteger( p_demux, "fake-id" );
-    p_sys->p_es_video = es_out_Add( p_demux->out, &fmt );
+    es_format_Init( &fmt_video, VIDEO_ES, VLC_FOURCC('f','a','k','e') );
+    fmt_video.i_id = var_CreateGetInteger( p_demux, "fake-id" );
+    p_sys->p_es_video = es_out_Add( p_demux->out, &fmt_video );
+
+    b_audio = var_CreateGetBool( p_demux, "audio" );
+
+    p_sys->video_period = (mtime_t)(1000000.0 / p_sys->f_fps);
+
+    if ( b_audio != false )
+    {
+        es_format_Init( &fmt_audio, AUDIO_ES, VLC_FOURCC('f','a','k','e') );
+        fmt_audio.audio.i_rate = var_CreateGetInteger( p_demux, "fake-rate" );
+
+        p_sys->p_es_audio = es_out_Add( p_demux->out, &fmt_audio );
+        p_sys->i_nb_samples = var_CreateGetInteger( p_demux, "fake-nb-audio-samples" );
+        p_sys->audio_period = (mtime_t)( 1000000.0 * p_sys->i_nb_samples / fmt_audio.audio.i_rate );
+        p_sys->period = p_sys->video_period < p_sys->audio_period ? p_sys->video_period : p_sys->audio_period;
+    }
+    else
+    {
+        p_sys->audio_period = 0;
+        p_sys->i_nb_samples = 0;
+        p_sys->period = p_sys->video_period;
+    }
 
     /* Update default_pts to a suitable value for access */
     var_Create( p_demux, "fake-caching", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
@@ -262,27 +305,59 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
 static int Demux( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
+    bool b_audio = false;
 
-    if ( !p_sys->i_last_pts )
+    if( p_sys->p_es_audio )
     {
-        p_sys->i_last_pts = p_sys->i_first_pts = mdate();
+        es_out_Control( p_demux->out, ES_OUT_GET_ES_STATE,
+                        p_sys->p_es_audio, &b_audio );
+    }
+
+    ////////////////////// PTS INCREMENTATION PART ////////////////
+    if ( !p_sys->i_last_pts || !p_sys->i_last_audio_pts )
+    {
+        p_sys->i_last_video_pts = p_sys->i_last_audio_pts =
+                                p_sys->i_last_pts = p_sys->i_first_pts = mdate();
+
         if ( p_sys->i_duration )
             p_sys->i_end_pts = p_sys->i_first_pts + p_sys->i_duration;
     }
     else
     {
-        p_sys->i_last_pts += (mtime_t)(1000000.0 / p_sys->f_fps);
+        p_sys->i_last_pts += p_sys->period;
         if ( p_sys->i_duration && p_sys->i_last_pts > p_sys->i_end_pts )
             return 0;
-        mwait( p_sys->i_last_pts );
+//        mwait( p_sys->i_last_pts );
     }
+    //////////////////////////////////////////////////////////////
+
+    es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_sys->i_last_pts );
+
 
-    block_t *p_block = block_New( p_demux, 1 );
-    p_block->i_flags |= BLOCK_FLAG_TYPE_I;
-    p_block->i_dts = p_block->i_pts = p_sys->i_last_pts;
+    ////////////////////////////////////// BLOCK CREATON PART/////
 
-    es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_block->i_pts );
-    es_out_Send( p_demux->out, p_sys->p_es_video, p_block );
+    //Video:
+    while ( p_sys->i_last_video_pts < p_sys->i_last_pts )
+    {
+        block_t *p_block = block_New( p_demux, 1 );
+        p_block->i_flags |= BLOCK_FLAG_TYPE_I;
+        p_block->i_dts = p_block->i_pts = p_sys->i_last_pts;
+        es_out_Send( p_demux->out, p_sys->p_es_video, p_block );
+
+        p_sys->i_last_video_pts += p_sys->video_period;
+    }
+    if ( b_audio != false )
+    {
+        while ( p_sys->i_last_audio_pts < p_sys->i_last_pts )
+        {
+            block_t* p_block_audio = block_New( p_demux, 1 );
+            p_block_audio->i_flags |= BLOCK_FLAG_TYPE_I;
+            p_block_audio->i_dts = p_block_audio->i_pts = p_sys->i_last_audio_pts;
+            es_out_Send( p_demux->out, p_sys->p_es_audio, p_block_audio );
+
+            p_sys->i_last_audio_pts += p_sys->audio_period;
+        }
+    }
 
     return 1;
 }
-- 
1.6.3.3

